﻿<#
.SYNOPSIS
	This script simulates the behavior of NUnit4PowerShell.dll
	It has been done to be able to debug inside the functions to test
#>
[CmdletBinding()]
param
(
	[string] $Filter = "*",
	[string] $Include = "",
	[string] $Exclude = "",
	[switch] $WhatIf
)

# This script path
Set-Variable -Name SCRIPT_LOCATION_PATH -Value (Split-Path (Resolve-Path $myInvocation.MyCommand.Path)) -Scope local

# NUnit4PowerShell config file name
Set-Variable -Name NUNIT4POWERSHELL_DLL_NAME -Value "NUnit4PowerShell.dll"

# NUnit4PowerShell config file name
Set-Variable -Name NUNIT4POWERSHELL_DLL_PATH -Value "$SCRIPT_LOCATION_PATH\$NUNIT4POWERSHELL_DLL_NAME"

function Load-NUnit4PowerShell
{
<#
.SYNOPSIS
	Load NUnit4PowerShell DLL
#>
	[CmdletBinding()]
	param()

	if (!(Test-Path $NUNIT4POWERSHELL_DLL_PATH))
	{
		throw "NUnit4PowerShell DLL has not been found at the following path : '$NUNIT4POWERSHELL_DLL_PATH'"
	}
	$dllLoading = [System.Reflection.Assembly]::LoadFile($NUNIT4POWERSHELL_DLL_PATH)
	Write-Verbose $dllLoading
}

function Execute-FixtureFunction
{
<#
.SYNOPSIS
	Execute a function contained in the current fixture executed (its functions are imported as module functions)
#>
	[CmdletBinding()]
	param
	(
		[string] $Name,
		[switch] $WhatIf
	)
	
	$function = Get-Item function:$Name -ErrorAction SilentlyContinue
	if ($function)
	{
		if ($WhatIf)
		{
			Write-Host "What if: Performing operation `"Execute FixtureFunction`" on Target `"Item: $Name`"".
			return
		}
		Invoke-Expression -Command  "$function" | Out-Null
	}
	
}

function Execute-PSFixture
{
<#
.SYNOPSIS
	Execute PSFixture written for NUnit4PowerShell
#>
	[CmdletBinding()]
	param
	(
		[string] $Path,
		[string] $Filter = "*",
		[string] $Include = "*",
		[string] $Exclude = "",
		[switch] $WhatIf
	)
	
	$moduleName = [System.IO.Path]::GetFileNameWithoutExtension($Path)
	Import-Module $Path -Force -DisableNameChecking | Out-Null

	Execute-FixtureFunction -Name "SetUp-TestFixture" -WhatIf:$WhatIf

	# Get the list of functions contains inside the fixture
	$functionsToTest = Get-Item function:$testFunctionNamePattern -Include $Include -Exclude $Exclude | Sort-Object -Property "Name"
	$functionsToTest = ($functionsToTest | Where-Object { $_.Name -like $Filter })
	
	foreach ($functionToTest in $functionsToTest)
	{
		Execute-FixtureFunction -Name "SetUp-Test" -WhatIf:$WhatIf

		if ($WhatIf)
		{
			Write-Host  "What if: Performing operation `"Execute Function`" on Target `"Item: $functionToTest`"".
		}
		else
		{
			try
			{
				Invoke-Expression -Command  "$functionToTest" | Out-Null
				Write-Host  "[Execute-PSTest] [OK]    $functionToTest"
			}
			catch [Exception]
			{
				Write-Host ("[Execute-PSTest] [ERROR] $functionToTest " + $_.Exception.ToString())
				continue
			}
		}
		Execute-FixtureFunction -Name "TearDown-Test" -WhatIf:$WhatIf
	}
	
	Execute-FixtureFunction -Name "TearDown-TestFixture" -WhatIf:$WhatIf
	
	Remove-Module -Name $moduleName -Force
}


#--------------------------------------------------------------------------------------------------------
# BEGINNING OF THE SCRIPT
#--------------------------------------------------------------------------------------------------------

Load-NUnit4PowerShell

$folderPath = [NUnit4PowerShell.Utils]::GetPSFixturesFolderPath()
Write-Verbose "The path of the folder containing the script is : '$folderPath'"
#Set-Location -Path $folderPath

$script:testFunctionNamePattern = [NUnit4PowerShell.Utils]::GetPSTestFunctionNamePattern()
$fixtureNamePattern = [NUnit4PowerShell.Utils]::GetPSFixtureNamePattern()

Write-Host "[Execute-PSFixtures] Launch tests"

$fixturePSFileList = Get-ChildItem -Path $folderPath -Filter $fixtureNamePattern -Recurse

foreach($fixture in $fixturePSFileList)
{
	Execute-PSFixture -Path $fixture.FullName -Include $Include -Exclude $Exclude -WhatIf:$WhatIf
}

Write-Host "[Execute-PSFixtures] Tests end"
